import Taro, { useLoad, useReady, useUnload } from '@tarojs/taro'
import { PropsWithChildren, useState } from 'react'
import { View, Video, Navigator, Image } from '@tarojs/components'
import useGetCustomClient from '@/hooks/useGetCustomClient'
import './index.less'

const WEB_VIEW_URL = 'plugin-private://wxf7261ed54f2e450e/pages/customPage/index'
const VIDEO_SRC = 'https://xycdn.yishihui.com/ad/prod/video/material_AD_1708595757_1708595757542.mp4'
const VIDEO_COVER = 'http://rescdn.yishihui.com/ad/prod/video/material_AD_1702267670_1702267670233.mp4?x-oss-process=video/snapshot,t_5000,f_jpg,w_0,h_0,m_fast'

// 在一些场景下 loadedMetaFiredInOnce 被触发了很多次
let loadedMetaFiredInOnce = 0
let durationTime = 0

const noop = () => {}

function PqCustom(props: PropsWithChildren<CustomPropsType>) {
  const {
    width, zIndex, onLoad = noop, onView = noop, onClick = noop, onClose = noop, onError = noop
  } = props
  const [show, setShow] = useState(true)

  function clickCustom() {
    
    closeCustom()
  }

  function closeCustom() {
    onClose()
    setShow(false)
  }

  function clickMask() {
    onClick()
    closeCustom()
  }

  return (
    <>
      {show && <View className='pq-custom' onClick={clickMask}>
        <View className='custom-container'>
          <View className='icon' style={{zIndex: (zIndex || 1) + 1}}>
            广告
          </View>
          <View className='close-icon'style={{zIndex: (zIndex || 1) + 1}} onClick={closeCustom}>
          </View>

          {/* <Navigator url={WEB_VIEW_URL}> */}
            <Custom
              width={width}
              zIndex={zIndex}
              onClick={clickCustom}
              onLoad={onLoad}
              onView={onView}
              onError={onError}
            />
          {/* </Navigator> */}
        </View>
      </View>}
    </>
  )
}

function Custom({ width, zIndex, onClick, onLoad, onView, onError }) {
  const [style, setStyle] = useState({ width: '0px', height: 'px', zIndex })

  useReady(() => {
    loadedMetaFiredInOnce = 0
    durationTime = 0

    observerAdCustom()
  })

  useGetCustomClient((customClient) => {
    let clientWidth = customClient.width
    let clientHeight = customClient.height
    if (width) {
      clientWidth = width
      clientHeight = width / 375 * 667
    }

    setStyle({
      zIndex: zIndex || 1,
      width: `${clientWidth}px`,
      height: `${clientHeight}px`
    })
  })

  // TODO: 监听无效 需要再想别的办法
  function observerAdCustom() {
    const viewObserver = Taro.createIntersectionObserver(PqCustom, { 
      thresholds: [0, 0.5]
    }).relativeToViewport()

    console.log(viewObserver)

    viewObserver.observe('#pq-custom', res => {
        const { intersectionRatio } = res
        if (intersectionRatio >= 0.5) {
          console.log('onView')
            onView()
            viewObserver.disconnect()
        }
    })
  }

  function onLoadedMetaData() {
    if (loadedMetaFiredInOnce++ < 1) {
      // 上报load
      onLoad()
    }
  }
  // 播放上报
  function onPlay() {
    console.log('onPlay')
  }

  // 播放进度，广告的整个生命周期中需要记录视频播放的进度
  function onTimeUpdate(res) {
    const { detail } = res
    durationTime += (detail?.currentTime || 0)
  }

  return (
    <View className='custom' style={style}>
      <Video id='pq-custom' className='custom-video' autoplay={true} controls={false} loop src={VIDEO_SRC} onClick={onClick} onLoadedMetaData={onLoadedMetaData} onPlay={onPlay} onTimeUpdate={onTimeUpdate} onError={onError} objectFit='fill' poster={VIDEO_COVER} />
      <View className='custom-bottom'>
        <Image src='http://rescdn.yishihui.com/ad/prod/image/logo_AD_1702267670_1702267670233.png' className='logo' />
        <View className='title'>5天站桩入门课</View>
        <View className='button'>免费领取</View>
      </View>
    </View>
  )
}

export default PqCustom
